#!/bin/bash

# This script determines the mpi install location based on either the env var
# OPAL_PREFIX or MPI_ROOT, or implicitly based on this script's location.
#
# If <location>/bin/orterun doesn't exist, we print a warning
# to note that this doesn't appear to be a legal MPI install
# at <location>.
#
# We use the install location to set
#     OPAL_PREFIX=<location>
#     MPI_ROOT=<location>
#     OPAL_LIBDIR=<location>/lib
#     LD_LIBRARY_PATH to contain <location>/lib
#     PMIX_INSTALL_PREFIX=<location>
#     HWLOC_PLUGINS_PATH=<location>/lib/hwloc
#
# Note: in the future the setting of OPAL_LIBDIR probably will become
# unnecessary. At the time this script was written, mpicc would produce
# the same -L<dir> regardless of the OPAL_PREFIX setting, so for now
# the safer option is to set OPAL_LIBDIR as well.


# ----------------------------------------------------------------
# Also add a default compiler selection for 'mpicc' etc.
# only set OMPI_CC/OMPI_CXX/OMPI_FC/OSHMEM_CC/OSHMEM_CXX/OSHMEM_FC if it's not already set
# one arg input: 'mpicc' etc
pick_default_compiler ()
{
    ppc=0
    if [ $(uname -m) = ppc64le ] ; then
        ppc=1
    fi
    if [ $1 = "${SMPI_CMD_PREFIX}cc" ] ; then
        complist="gcc:icc:ecc:pgcc:pathcc"
        if [ $ppc -eq 1 ] ; then
            complist="xlc_r:$complist"
        fi
        evar="${SMPI_WRAPPER_PREFIX}_CC"
    elif [ $1 = "${SMPI_CMD_PREFIX}c++" -o $1 = "${SMPI_CMD_PREFIX}CC" -o $1 = "${SMPI_CMD_PREFIX}cxx" ] ; then
        complist="g++:icc:ecc:pgCC:pathCC"
        if [ $ppc -eq 1 ] ; then
            complist="xlC_r:$complist"
        fi
        evar="${SMPI_WRAPPER_PREFIX}_CXX"
    elif [ $1 = 'mpif77' -o $1 = "${SMPI_CMD_PREFIX}fort" ] ; then
        complist="gfortran:ifort:ifc:efc:pgfortran:pgf90:pathf90"
        if [ $ppc -eq 1 ] ; then
            complist="xlf_r:$complist"
        fi
        evar="${SMPI_WRAPPER_PREFIX}_FC"
    elif [ $1 = 'mpif90' ] ; then
        complist="gfortran:ifort:ifc:pgfortran:pgf90:pathf90"
        if [ $ppc -eq 1 ] ; then
            complist="xlf90_r:$complist"
        fi
        evar=OMPI_FC
    else
        return
    fi

    eval "tmp=\${$evar:-''}"
    if [ ! -z "$tmp" ] ; then return ; fi

    found=0
    save_IFS=$IFS
    IFS=:
    for comp in $complist ; do
        for d in ${PATH:-''} ; do
            if [ $found -eq 0 -a -x "$d/$comp" ] ; then
                found=1
                usecomp=$comp
            fi
        done
    done
    if [ ! -z "$save_IFS" ] ; then
        IFS=$save_IFS
    else
        unset IFS
    fi

    if [ $found -ne 0 ] ; then
        eval "export $evar=$usecomp"
    fi
}

change_flags_xlf ()
{
    if [ $1 = 'mpif77' -o $1 = 'mpif90' -o $1 = "${SMPI_CMD_PREFIX}fort" ] ; then
	    INNER_FC=`basename "${!SMPI_EXT_FC}"`  # Deal with the full path of an XL compiler, also remove the leading space
	    if [ "${INNER_FC:0:2}" == "xl" ]; then
            if [ -z "${!SMPI_EXT_FCFLAGS}" ] ; then
                export $SMPI_EXT_FCFLAGS="-qthreaded -I${MPI_ROOT}/lib/XL "
            fi
        elif [ "${INNER_FC}" == "ifort" ]; then
            if [ -z "${!SMPI_EXT_FCFLAGS}" ] ; then
                export $SMPI_EXT_FCFLAGS="-pthread -I${MPI_ROOT}/lib/Intel "
            fi
        fi
    fi
}

change_flags_pgi()
{
    if [ $1 = "${SMPI_CMD_PREFIX}cc" ]; then
        INNER_CC=`basename "${!SMPI_EXT_CC}"`
        if [ "${INNER_CC}" = "pgcc" ]; then
           export $SMPI_EXT_CFLAGS="-lpthread"
        fi
    elif [ $1 = 'mpif77' -o $1 = 'mpif90' -o $1 = "${SMPI_CMD_PREFIX}fort" ]; then
        INNER_FC=`basename "${!SMPI_EXT_FC}"`
        if [ "${INNER_FC:0:3}" = "pgf" ]; then
           export $SMPI_EXT_FCFLAGS="-lpthread -I${MPI_ROOT}/lib/PGI "
        fi
    elif [ $1 = "${SMPI_CMD_PREFIX}c++" -o $1 = "${SMPI_CMD_PREFIX}CC" -o $1 = "${SMPI_CMD_PREFIX}cxx" ]; then
        INNER_CXX=`basename "${!SMPI_EXT_CXX}"`
        if [ "${INNER_CXX}" = "pgCC" -o "${INNER_CXX}" = "pgc++" ]; then
           export $SMPI_EXT_CXXFLAGS="-lpthread"
        fi
    fi
} 

change_flags_nvcc ()
{
    if [ $1 = "${SMPI_CMD_PREFIX}cc" ] ; then
	INNER_CC=`basename "${!SMPI_EXT_CC}"`  # Deal with the full path of compiler, also remove the leading space
	if [ "${INNER_CC:0:4}" == "nvcc" ]; then
            if [ -z "${!SMPI_EXT_CFLAGS}" ] ; then
                export $SMPI_EXT_CFLAGS=" "
            fi
        fi
    elif [ $1 = "${SMPI_CMD_PREFIX}c++" -o $1 = "${SMPI_CMD_PREFIX}CC" -o $1 = "${SMPI_CMD_PREFIX}cxx" ] ; then
	INNER_CXX=`basename "${!SMPI_EXT_CXX}"`  # Deal with the full path of compiler, also remove the leading space
	if [ "${INNER_CXX:0:4}" == "nvcc" ]; then
            if [ -z "${!SMPI_EXT_CXXFLAGS}" ] ; then
                export $SMPI_EXT_CXXFLAGS=" "
            fi
        fi
    fi
}

location=$(dirname "$0")
this=$(basename "$0")

SMPI_WRAPPER_PREFIX=OMPI
SMPI_CMD_PREFIX=mpi
if [[ $this = osh* ]] ; then
    SMPI_WRAPPER_PREFIX=OSHMEM
    SMPI_CMD_PREFIX=osh
fi

SMPI_EXT_FC="${SMPI_WRAPPER_PREFIX}_FC"
SMPI_EXT_CC="${SMPI_WRAPPER_PREFIX}_CC"
SMPI_EXT_CXX="${SMPI_WRAPPER_PREFIX}_CXX"
SMPI_EXT_CFLAGS="${SMPI_WRAPPER_PREFIX}_CFLAGS"
SMPI_EXT_CXXFLAGS="${SMPI_WRAPPER_PREFIX}_CXXFLAGS"
SMPI_EXT_FCFLAGS="${SMPI_WRAPPER_PREFIX}_FCFLAGS"

mpiroot=${OPAL_PREFIX:-""}
if [ -z "$mpiroot" ] ; then
    mpiroot=${MPI_ROOT:-""}
fi
if [ -z "$mpiroot" ] ; then
    mpiroot=$(cd "$location/.." ; pwd)
else
# Check if the location of this script is equal to the chosen mpiroot (which
# at this point came from either OPAL_PREFIX or MPI_ROOT)
  if [ $(cd "$mpiroot" ; pwd) != $(cd "$location/.." ; pwd) ] ; then
    echo "Warning: location of $this differs from OPAL_PREFIX / MPI_ROOT."
    echo "$this at $location"
    echo "OPAL_PREFIX=$OPAL_PREFIX"
    echo "MPI_ROOT=$MPI_ROOT"
  fi
fi

if [ ! -x ${mpiroot}/bin/orterun ] ; then
    echo "Warning: ${mpiroot} may not be a full MPI installation."
fi
if [ ! -z ${OPAL_PREFIX:-""} -a ! -z ${MPI_ROOT:-""} ] ; then
  if [ ${OPAL_PREFIX:-""} != ${MPI_ROOT:-""} ] ; then
    echo "Warning: OPAL_PREFIX and MPI_ROOT set to different values."
    echo "OPAL_PREFIX=$OPAL_PREFIX"
    echo "MPI_ROOT=$MPI_ROOT"
  fi
fi

export OPAL_PREFIX="$mpiroot"
export MPI_ROOT="$mpiroot"

# only set OPAL_LIBDIR if one's not already set
if [ -z "$OPAL_LIBDIR" ] ; then
  export OPAL_LIBDIR="$mpiroot/lib"
fi

if [ -z "$PMIX_INSTALL_PREFIX" ] ; then
  export PMIX_INSTALL_PREFIX="$mpiroot"
fi
if [ -z "$HWLOC_PLUGINS_PATH" ] ; then
  export HWLOC_PLUGINS_PATH="$mpiroot/lib/hwloc"
fi

if [ -z "$LD_LIBRARY_PATH" ] ; then
    export LD_LIBRARY_PATH="$mpiroot/lib"
else
# See if $mpiroot/lib is already contained in LD_LIBRARY_PATH before adding it.
# (without relying on pure string comparison for directories)
    new=$(cd $mpiroot/lib 1>/dev/null 2>&1 ; pwd -P)
    found=0
    save_IFS=$IFS
    IFS=:
    for d in ${LD_LIBRARY_PATH:-''} ; do
        if [ "$(cd $d 1>/dev/null 2>&1 ; pwd -P)" = "$new" ] ; then
            found=1
        fi
    done
    if [ ! -z "$save_IFS" ] ; then
        IFS=$save_IFS
    else
        unset IFS
    fi
    if [ $found -eq 0 ] ; then
        export LD_LIBRARY_PATH="$mpiroot/lib:${LD_LIBRARY_PATH}"
    fi
fi

if [ $this = 'mpirun' -o $this = 'mpiexec' -o $this = 'orterun' -o $this = 'oshrun' ] ; then
    exec $location/alias.pl "$location/stock/$this" "$@"
elif [ $this = 'ompi_info' ] ; then
    # To resolve the libpami.so we need to point to the location of the portable PAMI
    # and we need to LD_PRELOAD the libpami_cudahook.so
    export LD_LIBRARY_PATH="${OPAL_PREFIX}/lib/pami_port/:${LD_LIBRARY_PATH}"
    if [ -z "$LD_PRELOAD" ] ; then
        export LD_PRELOAD="${OPAL_LIBDIR}/libpami_cudahook.so"
    else
        export LD_PRELOAD="${OPAL_LIBDIR}/libpami_cudahook.so:${LD_PRELOAD}"
    fi

    # include very simple parse for -restrict_libs exclusively in first arg:
    if [ "$1" = "-restrict_libs" ] ; then
        export OMPI_MCA_mpi_restrict_libs="$2"
        shift ; shift
    fi

    exec "$location/stock/$this" "$@"
else
    pick_default_compiler $this
    change_flags_xlf $this
    change_flags_nvcc $this
    change_flags_pgi $this
    exec "$location/stock/$this" "$@"
fi
