/*
 *  Copyright (c) 1997-2016 IBM Corporation.  All rights reserved.
 *
 *	 Function:	- example: ping-pong benchmark
 *
 *	 Usage:		mpirun -np 2 ping_pong [nbytes]
 *
 *	 Notes:		- Define CHECK macro to check data integrity.
 *			- The previous ping_pong example timed each
 *			  iteration.  The resolution of MPI_Wtime() is
 *			  not sufficient to provide accurate measurements
 *			  when nbytes is small.  This version times the
 *			  entire run and reports average time to avoid
 *			  this issue.
 *			- To avoid cache conflicts due to direct
 *			  process-to-process bcopy, the buffers are
 *			  aligned and offset from each other.
 *			- Use of direct process-to-process bcopy coupled
 *			  with the fact that the data is never touched
 *			  results in inflated bandwidth numbers when
 *			  nbytes <= cache size.  To obtain a more
 *			  representative bandwidth measurement, increase
 *			  nbytes to at least 2*cache size (2MB).
 *       - For good latency on the same node, try binding
 *         the ranks as close together as possible (not the
 *         default at this time)
 *         --report-bindings --bind-to core --map-by core -prot
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <mpi.h>

#define NLOOPS		1000
#define ALIGN		4096

int
main(int argc, char *argv[])
{
	int		i;
#ifdef CHECK
	int		j;
#endif
	double		start, stop;
	int		nbytes = 0;
	int		rank, size;
	MPI_Status	status;
	char		*buf;

	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &size);

	if (size != 2) {
		if ( ! rank) printf("ping_pong: must have two processes\n");
		MPI_Finalize();
		exit(0);
	}

	nbytes = (argc > 1) ? atoi(argv[1]) : 0;
	if (nbytes < 0) nbytes = 0;
/*
 * Page-align buffers and displace them in the cache to avoid collisions.
 */
	buf = (char *) malloc(nbytes + 524288 + (ALIGN - 1));
	if (buf == 0) {
		MPI_Abort(MPI_COMM_WORLD, MPI_ERR_BUFFER);
		exit(1);
	}

	buf = (char *) ((((MPI_Aint) buf) + (ALIGN - 1)) & ~(ALIGN - 1));
	if (rank == 1) buf += 524288;
	memset(buf, 0, nbytes);
/*
 * Ping-pong.
 */
	if (rank == 0) {
		printf("ping-pong %d bytes ...\n", nbytes);

/*
 * warm-up loop
 */
		for (i = 0; i < 5; i++) {
			MPI_Send(buf, nbytes, MPI_CHAR, 1, 1, MPI_COMM_WORLD);
			MPI_Recv(buf, nbytes, MPI_CHAR,
					1, 1, MPI_COMM_WORLD, &status);
		}
/*
 * timing loop
 */
		start = MPI_Wtime();
		for (i = 0; i < NLOOPS; i++) {
#ifdef CHECK
			for (j = 0; j < nbytes; j++) buf[j] = (char) (j + i);
#endif
			MPI_Send(buf, nbytes, MPI_CHAR,
					1, 1000 + i, MPI_COMM_WORLD);
#ifdef CHECK
			memset(buf, 0, nbytes);
#endif
			MPI_Recv(buf, nbytes, MPI_CHAR,
					1, 2000 + i, MPI_COMM_WORLD, &status);
#ifdef CHECK
			for (j = 0; j < nbytes; j++) {
				if (buf[j] != (char) (j + i)) {
					printf("error: buf[%d] = %d, not %d\n",
						j, buf[j], j + i);
					break;
				}
			}
#endif
		}
		stop = MPI_Wtime();

		printf("%d bytes: %.2f usec/msg\n",
			nbytes, (stop - start) / NLOOPS / 2 * 1000000);
		if (nbytes > 0) {
			printf("%d bytes: %.2f MB/sec\n", nbytes,
				nbytes / 1000000. /
				((stop - start) / NLOOPS / 2));
		}
	}
	else {
/*
 * warm-up loop
 */
		for (i = 0; i < 5; i++) {
			MPI_Recv(buf, nbytes, MPI_CHAR,
					0, 1, MPI_COMM_WORLD, &status);
			MPI_Send(buf, nbytes, MPI_CHAR, 0, 1, MPI_COMM_WORLD);
		}

		for (i = 0; i < NLOOPS; i++) {
			MPI_Recv(buf, nbytes, MPI_CHAR,
					0, 1000 + i, MPI_COMM_WORLD, &status);
			MPI_Send(buf, nbytes, MPI_CHAR,
					0, 2000 + i, MPI_COMM_WORLD);
		}
	}

	MPI_Finalize();
	exit(0);
}
